import React from 'react';
import { RenderValueProp } from './DropdownListInput';
import { BaseListboxInputProps, Filterable, PopupWidgetProps, Searchable, WidgetHTMLProps, WidgetProps } from './shared';
import { DataItem, WidgetHandle } from './types';
export declare type DropdownHandle = WidgetHandle;
export interface DropdownProps<TDataItem> extends WidgetProps, WidgetHTMLProps, PopupWidgetProps, Searchable, Filterable<TDataItem>, BaseListboxInputProps<TDataItem> {
    name?: string;
    autoFocus?: boolean;
    autoComplete?: 'on' | 'off';
    onCreate?: (searchTerm: string) => void;
    renderValue?: RenderValueProp<TDataItem>;
}
declare interface DropdownList {
    <TDataItem = DataItem>(props: DropdownProps<TDataItem> & React.RefAttributes<DropdownHandle>): React.ReactElement | null;
    displayName?: string;
    propTypes?: any;
}
/**
 * ---
 * shortcuts:
 *   - { key: alt + down arrow, label: open dropdown }
 *   - { key: alt + up arrow, label: close dropdown }
 *   - { key: down arrow, label: move focus to next item }
 *   - { key: up arrow, label: move focus to previous item }
 *   - { key: home, label: move focus to first item }
 *   - { key: end, label: move focus to last item }
 *   - { key: enter, label: select focused item }
 *   - { key: ctrl + enter, label: create new option from current searchTerm }
 *   - { key: any key, label: search list for item starting with key }
 * ---
 *
 * A `<select>` replacement for single value lists.
 * @public
 */
declare const DropdownList: DropdownList;
export default DropdownList;
//# sourceMappingURL=DropdownList.d.ts.map