import PropTypes from 'prop-types';
import React from 'react';
import ListOption, { ListOptionProps } from './ListOption';
import { UserProvidedMessages } from './messages';
import { WidgetHTMLProps } from './shared';
import { DataItem, RenderProp, Value } from './types';
import { DataKeyAccessor, TextAccessor } from './util/dataHelpers';
export declare type GroupBy<TDataItem> = string | ((item: TDataItem) => string);
export interface ListboxHandle {
    scrollIntoView(): void;
}
export declare type RenderItemProp<TDataItem> = RenderProp<{
    item: TDataItem;
    searchTerm?: string;
    index: number;
    text: string;
    value: unknown;
    disabled: boolean;
}>;
export declare type RenderGroupProp = RenderProp<{
    group: string;
}>;
export declare type OptionComponentProp = React.ComponentType<ListOptionProps<any>>;
interface BaseListboxProps<TDataItem> extends WidgetHTMLProps {
    bordered?: boolean;
    data: TDataItem[];
    focusedItem?: TDataItem;
    className?: string;
    disabled?: boolean | TDataItem[];
    messages?: UserProvidedMessages;
    textField?: TextAccessor;
    dataKey?: DataKeyAccessor;
    renderItem?: RenderItemProp<TDataItem>;
    renderGroup?: RenderGroupProp;
    activeId?: string;
    searchTerm?: string;
    groupBy?: GroupBy<TDataItem>;
    optionComponent?: OptionComponentProp;
    onMove?(selectedElement: Element, listElement: HTMLDivElement, focusedItem: TDataItem): void;
}
declare type SingleChangeHandler<TDataItem> = (dataItem: TDataItem, metadata: {
    lastValue: Value;
    originalEvent?: React.SyntheticEvent;
}) => void;
interface SingleListboxProps<TDataItem> extends BaseListboxProps<TDataItem> {
    value?: Value;
    defaultValue?: Value;
    multiple?: false;
    onChange?: SingleChangeHandler<TDataItem>;
}
declare type MultipleChangeHandler<TDataItem> = (dataItem: TDataItem[], metadata: {
    action: 'insert' | 'remove';
    dataItem: TDataItem;
    lastValue: Value;
    originalEvent?: React.SyntheticEvent;
}) => void;
interface MultipleListboxProps<TDataItem> extends BaseListboxProps<TDataItem> {
    value?: Value[];
    defaultValue?: Value[];
    multiple: true;
    onChange?: MultipleChangeHandler<TDataItem>;
}
export declare type ListboxProps<TDataItem> = SingleListboxProps<TDataItem> | MultipleListboxProps<TDataItem>;
declare interface Listbox {
    <TDataItem = DataItem>(props: ListboxProps<TDataItem> & React.RefAttributes<ListboxHandle>): React.ReactElement | null;
    displayName?: string;
}
declare const Listbox: Listbox;
declare const _default: Listbox & {
    defaultProps: {
        onSelect: () => void;
        data: never[];
        optionComponent: typeof ListOption;
    };
    propTypes: {
        data: PropTypes.Requireable<any[]>;
        dataKey: any;
        textField: any;
        onSelect: PropTypes.Requireable<(...args: any[]) => any>;
        onMove: PropTypes.Requireable<(...args: any[]) => any>;
        onHoverOption: PropTypes.Requireable<(...args: any[]) => any>;
        activeId: PropTypes.Requireable<string>;
        optionComponent: any;
        renderItem: PropTypes.Requireable<(...args: any[]) => any>;
        renderGroup: PropTypes.Requireable<(...args: any[]) => any>;
        focusedItem: PropTypes.Requireable<any>;
        selectedItem: PropTypes.Requireable<any>;
        searchTerm: PropTypes.Requireable<string>;
        disabled: any;
        messages: PropTypes.Requireable<PropTypes.InferProps<{
            emptyList: PropTypes.Validator<(...args: any[]) => any>;
        }>>;
    };
};
export default _default;
//# sourceMappingURL=Listbox.d.ts.map